#!/usr/bin/perl

use strict;
use warnings;
use Data::Dumper;
use File::Basename;

my $filename = shift;
my $result = search( $filename );
$result->{filename} = basename( $filename );
foreach my $key ( keys %$result ) {
  print "$key : $result->{$key}\n";
}
exit;

sub search {
  my $filename = shift;
  # print "Filename: $filename\n";
  my $data;
  # バージョン検索（1回目）：初回16KB読んでみる
  my $result = search_version( read_header( $filename ) );    
  # 16KBだと判断できない場合があるので、判断
  if ( !exists $result->{documentversion} ) {
    # バージョン検索（2回目）：全部読む（AICS2,CS3のEPS）
    $result = search_version( read_all( $filename ) );    
  } 
  return $result;
} 

sub search_version {
  my $data = shift;
  my $result = {};
  # 1行目の判定
  ( my $first = $data ) =~s/\n.+$//sgo;
  if ( $first =~ m/%PDF-1.\d/ ) {
    $result->{type} = 'aipdf';
  }
  elsif ( $first =~ m/\xC5\xD0\xD3\xC6/ ) {
    $result->{type} = 'eps';
    $result->{preview} = 'tiff';
  }
  elsif ( $first =~ m/%!PS-Adobe-/ ) {
    $result->{type} = 'eps';
  }
  # バージョン文字列検索
  my $count;
  foreach ( split /\n/, $data ) {
    # %%Creator：使用Illustratorバージョンの判定
    if ( m/%%Creator: *(.+)/ ) {
      # Illustrator CS2以降のEPSはあとの%%Creatorが正解
      if ( exists $result->{type} && $result->{type} eq 'eps') {
        if ( $1 =~ m/(Adobe Illustrator\(R\) 12)$/ )   {
		  $count++;
	      if ( $count == 1 ) {
		  }
		  else {
		    $result->{documentversion} = $1;
	      }
          next;
        }
        elsif ( $1 =~ m/(Adobe Illustrator\(R\) 1\d.\d$)/ )   {
		  $count++;
	      if ( $count == 1 ) {
		  }
		  else {
		    $result->{documentversion} = $1;
	      }
          next;
        }
        else {
          $result->{documentversion} = $1;
        }
      }
      else {
        $result->{documentversion} = $1;
      }
    }
    # %%AI8_CreatorVersion：保存バージョン
    elsif ( m/%%AI8_CreatorVersion: *(.+)/ ) { 
      $result->{creatorversion} = $1;
    }
    # バージョン文字列が揃ったら探索終了
    if ( $result->{documentversion} 
      && $result->{creatorversion} ) {
      last;
    }
  }
  return $result;
}

sub read_header {
  my $filename = shift;
  my $data;
  {
    open my $fh, '<', $filename or die $!;
    read $fh, $data, 16384;
    close $fh;
  }
    # 改行コード混在しているので統一
  $data =~ s/[\x0d\x0a|\x0a|\x0d]/\n/g;
  return $data;
}

sub read_all {
  my $filename = shift;
  my $data;
  {
    open my $fh, '<', $filename or die $!;
    $data .= do { local $/; <$fh> };
    close $fh
  }
  # 改行コード混在しているので統一
  $data =~ s/[\x0d\x0a|\x0a|\x0d]/\n/g;
  return $data;
}